@@ -8,7 +8,8 @@ use core::{
 use serde::{Deserialize, Serialize};
 
 use crate::{
-    slab_arc::SlabArc,
+    byte_slab::BSlab,
+    slab_arc::{SlabArc, RerooterKey},
     slab_slice_arc::{SlabSliceArc, SlabStrArc}
 };
 
@@ -102,22 +103,38 @@ impl<'a, const N: usize, const SZ: usize> ManagedArcSlab<'a, N, SZ> {
         ManagedArcSlab::Owned(arc.clone())
     }
 
-    pub fn reroot(self, arc: &SlabArc<N, SZ>) -> Option<ManagedArcSlab<'static, N, SZ>> {
+    pub fn reroot_with_key(self, key: &RerooterKey) -> Option<ManagedArcSlab<'static, N, SZ>> {
         match self {
             ManagedArcSlab::Owned(e) => Some(ManagedArcSlab::Owned(e)),
             ManagedArcSlab::Borrowed(b) => {
-                if arc.is_empty() || b.is_empty() {
+                if b.is_empty() {
                     // TODO: nuance
                     return None;
                 }
 
                 // TODO: yolo ub
-                let start: usize = arc.deref().as_ptr() as usize;
-                let end: usize = start + arc.deref().len();
+                let start: usize = key.start as usize;
+                let end: usize = key.end as usize;
                 let b_start: usize = b.as_ptr() as usize;
 
                 if (start <= b_start) && (b_start < end) {
-                    let ssa = arc.sub_slice_arc(b_start - start, b.len()).ok()?;
+                    let bslab: &'static BSlab<N, SZ> = unsafe { &*key.slab.cast::<BSlab<N, SZ>>() };
+
+                    // NOTE: We *don't* increase the refcount for this arc, as it technically aliases the
+                    // borrowed one that created the `key` here. Since we will forget this arc after
+                    // creating the ssa, don't update!
+                    let arc = SlabArc {
+                        slab: bslab,
+                        idx: key.idx,
+                    };
+
+                    let ssa = arc
+                        .sub_slice_arc(b_start - start, b.len())
+                        .ok()?;
+
+                    // Okay, now forget that arc ever happened
+                    core::mem::forget(arc);
+
                     Some(ManagedArcSlab::Owned(ssa))
                 } else {
                     None
@@ -137,6 +154,24 @@ pub enum ManagedArcStr<'a, const N: usize, const SZ: usize> {
     Owned(SlabStrArc<N, SZ>),
 }
 
+
+impl<'a, const N: usize, const SZ: usize> PartialEq<str> for ManagedArcStr<'a, N, SZ> {
+    fn eq(&self, other: &str) -> bool {
+        let stir: &str = self.deref();
+        stir.eq(other)
+    }
+}
+
+impl<'a, const N: usize, const SZ: usize> PartialEq for ManagedArcStr<'a, N, SZ> {
+    fn eq(&self, other: &Self) -> bool {
+        let stir_me: &str = self.deref();
+        let stir_ot: &str = other.deref();
+        stir_me.eq(stir_ot)
+    }
+}
+
+impl<'a, const N: usize, const SZ: usize> Eq for ManagedArcStr<'a, N, SZ> { }
+
 impl<'a, const N: usize, const SZ: usize> defmt::Format for ManagedArcStr<'a, N, SZ> {
     fn format(&self, fmt: defmt::Formatter<'_>) {
         self.deref().format(fmt)
@@ -212,26 +247,40 @@ impl<'a, const N: usize, const SZ: usize> ManagedArcStr<'a, N, SZ> {
         ManagedArcStr::Owned(arc.clone())
     }
 
-    pub fn reroot(self, arc: &SlabArc<N, SZ>) -> Option<ManagedArcStr<'static, N, SZ>> {
+    pub fn reroot_with_key(self, key: &RerooterKey) -> Option<ManagedArcStr<'static, N, SZ>> {
         match self {
             ManagedArcStr::Owned(e) => Some(ManagedArcStr::Owned(e)),
             ManagedArcStr::Borrowed(b) => {
-                if arc.is_empty() || b.is_empty() {
+                if b.is_empty() {
                     // TODO: nuance
                     return None;
                 }
 
                 // TODO: yolo ub
-                let start: usize = arc.deref().as_ptr() as usize;
-                let end: usize = start + arc.deref().len();
+                let start: usize = key.start as usize;
+                let end: usize = key.end as usize;
                 let b_start: usize = b.as_ptr() as usize;
 
                 if (start <= b_start) && (b_start < end) {
+                    let bslab: &'static BSlab<N, SZ> = unsafe { &*key.slab.cast::<BSlab<N, SZ>>() };
+
+                    // NOTE: We *don't* increase the refcount for this arc, as it technically aliases the
+                    // borrowed one that created the `key` here. Since we will forget this arc after
+                    // creating the ssa, don't update!
+                    let arc = SlabArc {
+                        slab: bslab,
+                        idx: key.idx,
+                    };
+
                     let ssa = arc
                         .sub_slice_arc(b_start - start, b.len())
                         .ok()?
                         .into_str_arc()
                         .ok()?;
+
+                    // Okay, now forget that arc ever happened
+                    core::mem::forget(arc);
+
                     Some(ManagedArcStr::Owned(ssa))
                 } else {
                     None
@@ -240,3 +289,86 @@ impl<'a, const N: usize, const SZ: usize> ManagedArcStr<'a, N, SZ> {
         }
     }
 }
+
+/// A trait for restoring Managed types to a given SlabArc
+///
+/// For types that are not Managed, this trait should just return
+/// the type as-is.
+pub trait Reroot
+{
+    /// The return type. This is used to change the lifetimes, e.g. to convert
+    /// `Self: ManagedArcSlab<'a, _, _>` to `Retval: ManagedArcSlab<'static, _, _>`.
+    type Retval;
+
+    fn reroot(self, key: &RerooterKey) -> Result<Self::Retval, ()>;
+}
+
+impl<'a, const N: usize, const SZ: usize> Reroot for ManagedArcSlab<'a, N, SZ> {
+    type Retval = ManagedArcSlab<'static, N, SZ>;
+
+    fn reroot(self, key: &RerooterKey) -> Result<Self::Retval, ()>
+    {
+        self.reroot_with_key(key).ok_or(())
+    }
+}
+
+impl<'a, const N: usize, const SZ: usize> Reroot for ManagedArcStr<'a, N, SZ> {
+    type Retval = ManagedArcStr<'static, N, SZ>;
+
+    fn reroot(self, key: &RerooterKey) -> Result<Self::Retval, ()>
+    {
+        self.reroot_with_key(key).ok_or(())
+    }
+}
+
+macro_rules! reroot_nop {
+    (
+        [$($rrty:ty),+]
+    ) => {
+        $(
+            impl Reroot for $rrty {
+                type Retval = $rrty;
+
+                #[inline(always)]
+                fn reroot(self, _key: &RerooterKey) -> Result<Self::Retval, ()>
+                {
+                    Ok(self)
+                }
+            }
+        )+
+    };
+}
+
+reroot_nop!([u8, u16, u32, u64]);
+reroot_nop!([i8, i16, i32, i64]);
+reroot_nop!([bool, char, ()]);
+reroot_nop!([f32, f64]);
+
+#[cfg(test)]
+mod test {
+    use crate::{BSlab, ManagedArcSlab, Reroot};
+    use std::ops::Deref;
+
+    #[test]
+    fn smoke() {
+        static SLAB: BSlab<4, 128> = BSlab::new();
+        SLAB.init().unwrap();
+
+        let mut sbox = SLAB.alloc_box().unwrap();
+
+        sbox[..4].copy_from_slice(&[1, 2, 3, 4]);
+
+        let arc_1 = sbox.into_arc();
+
+        let brw = ManagedArcSlab::<4, 128>::Borrowed(&arc_1[..4]);
+        let own: ManagedArcSlab<'static, 4, 128> = brw.reroot(&arc_1).unwrap();
+
+        match own {
+            ManagedArcSlab::Owned(ssa) => {
+                assert_eq!(&[1, 2, 3, 4], ssa.deref());
+            }
+            _ => panic!("Not owned!"),
+        }
+    }
+
+}
